# 'akrontab' starter - triggered by master crontab server    
# Author: R.Toebbicke, CERN    
# Kerberized: bjac3    
#    
# to be 'require'd by arcd.arc    

$ACL{'akron'}="acron";
    
use vars qw($ARCLOG $USE_SYSLOG $proc $args $opt_t $opt_u $opt_d $Debug $reply $uid $krb5file);
use Fcntl;
use File::Temp;
use lib "/usr/share/perl5/acron/";
use Heimtools;
undef $USE_SYSLOG;
$ARCLOG = (-d "/var/log") ? "/var/log/$proc.log" : "/var/adm/$proc.log";

use strict;
1;

sub akron {
    &safeargs($args);
    &Log("$proc start $args");
    $opt_t=0 ;
    
# command format: akron -u<userid> [-t] [-d]
    $args=&optargs($args);
    $Debug = $opt_d;
    
    &Die("Unsecured channel") unless $ENV{'SCPRIVATE'};         
    &Die("No X-realm support") unless $ENV{'SCREALM'} eq $ENV{'SCLREALM'};
    my $TOKENS   = _locate("tokens"  , "/usr/bin");
    my $PTS      = _locate("pts"     , "/usr/bin");
    my $chown    = _locate("chown"   , "/bin",         "/usr/bin");
    my ($kinit);


# Check whether user is allowed to execute cron jobs here        
    foreach my $cron ("/var/adm/cron", "/etc/cron.d", "/usr/lib/cron") {
        next unless -d $cron;        
        foreach my $type ("allow", "deny") {
            my $f="$cron/cron.$type";
            next unless -e $f;
            open(C,$f) || &Die("Unable to access $f: $!");
            my @allow=<C>;
            close(C);
            my $exists=grep(/^$opt_u$/,@allow);
            if (($exists && $type eq "deny") ||
                ((!$exists) && $type eq "allow")) {
                &Die("Cron access denied for '$opt_u'") ;
            }
            last;
        }        
        last;        
    }        


# verify user is in password file        
    my @pw = getpwnam($opt_u);
    &Die("User $opt_u not defined on this machine") unless @pw;
    $uid = $pw[2];                      # unix id, hopefully afs id
    my ($afsid) = `$PTS exam $opt_u -noauth` =~ /$opt_u,\s+id:\s+(\d+),\s+/;
    &Die("Strange definition for user $opt_u. unix ID $uid, AFS ID $afsid") if $uid != $afsid;
        

# always ask for a kerberos5 ticket
    if ( ($kinit    = _locate("+", "kinit", "/usr/bin"))
       && (-e "/etc/krb5.conf") ) {
	  $krb5file = File::Temp::tempnam ('/tmp', 'krb5_acron-_XXXXXXXX');
          $ENV{'KRB5CCNAME'} = "FILE:$krb5file";
    } else {
	&Die("arrived on a non-Kerberos5 machine - does not work");
    }

# our caller is waiting for us: send back an aknowlegment        
    # and tell it we want complete kerberos tickets (could also ask for token only).
    my $answer = "K";   # A" would have been AFS-token only
    syswrite(STDOUT, "<OK>$answer", 5);
    &Log("replied $answer to caller");
        
        
# now we wait for either a token or a composite kerberos5 ticket        
# immediately followed by the command text.        
    my ($token, $krb5ticket, $reply) = &_read_ans(*STDIN);

# then set token & kerberos ticket
    my $pid;
    if (($pid = fork()) == -1) { die "error forking a child"; }
    elsif ($pid  == 0 ) {     # child
      local ($<, $>);
      $< = $> = $uid;         # run as uid=user, euid=user in child

      if ($krb5ticket) {
	  unlink $krb5file;
	  # if someone tries to out-race us, the sysopen in Do_kinit will fail
	  &__Do_kinit($kinit, $krb5ticket, $krb5file);
      } else {
	  &Die("no krb5 ticket returned by server");
      }
      exit 0;
    }
    else {
      waitpid($pid,0);
      &Log("rc from child is $?");
    }

    # print token
    &Log(`$TOKENS 2>&1`) if $Debug;        #! warning: goes through a shell
        
        
# verify user is in password file, part 2
    $ENV{'HOME'}=$pw[7];        
    &Die("Unknown user '$opt_u' on $ENV{SCLHOST}: '$ENV{HOME}' is not a directory") unless -d $ENV{'HOME'};        
    chdir $ENV{'HOME'} || &Die("Cannot chdir $ENV{HOME}");
# danger from now, since user environment is visible

            
# now, the command itself        
    if ($opt_t) {           # checking only        
        print "OK\n";        
        &Log("$proc tested OK");        
        &clean;
        return;        
    }        


# at this point, we let arcd disconnect from caller & start an independant life
    close(STDIN); close(STDOUT); close(STDERR);

    # bind EOS FUSE credentials if applicable
    
     if (-x "/usr/bin/eosfusebind") 
     {
       $reply = "/usr/bin/eosfusebind;" . $reply
     }

    &Log("command: $reply");
#   my @input = split(/\n/, $reply);
        
    $ENV{'SHELL'}="/bin/sh";        
    $ENV{'LOGNAME'}=$opt_u;        
    delete $ENV{'BASH_ENV'};      # some SLC4 nodes have /root/.bashrc read-protected
    delete $ENV{'ENV'};
        
    my ($TMP, $tmp);
    ($TMP, $tmp) = File::Temp::tempfile("/tmp/${proc}XXXXXX");
    &Log("\$tmp now: $tmp, fileno(\$TMP)=".fileno($TMP));
    # here, modern perl would accept (STDOUT,">&",$TMP) , but then older versions would fail
    # alternatively, old versions of perl would accept (STDOUT,">&$Tmp), rejected by modern ones
    open(STDOUT,">&".fileno($TMP)) or die $!;
    open(STDERR,">&".fileno($TMP)) or die $!;

    delete $ENV{'BASH_ENV'};      # some SLC4 nodes have /root/.bashrc read-protected
    delete $ENV{'ENV'};
    my $f_flag = ($pw[8]=~/csh$/) ? "f" : "";
    my $f_run = "/bin/su $opt_u -${f_flag}c";
    $f_run .= " $ENV{'SHELL'}";

    ($f_run) = $f_run =~ /^(.+)$/;     # wash it
    &Log("shall now run '$f_run' with args '$reply'");
    open(J,"|$f_run") ;
    print J $reply;
    close(J);        
    &Log("$proc end, rc=$?");        
    close(STDOUT); close(STDERR);       # flush. (probably useless in parent)

    my @S = stat($TMP);
    if ($S[7]) {                        # is size non zero?
        (my $mailuser = $opt_u) =~ s/^_//;                           # trick for special users
        my $now = localtime(time());
        my $sub = "Cron job output from $ENV{SCLHOST}:$mailuser at $now";
        my $minus_s="-s '$sub'";
        # Sendmail sets Reply-To to norely@cern.ch ensuring noreply messages are not sent to root@machine and sit on a queue
        if (-e "/usr/sbin/sendmail")
        {
            open(M,"|/usr/sbin/sendmail -fnoreply\@cern.ch $mailuser\@mail.cern.ch");
            print M qq|From: root <root\@$ENV{SCLHOST}>\n|;
            print M "To: $mailuser\@cern.ch\n";
            print M "Subject: $sub\n";
        }
        else
        {
            my $open = "/bin/mail $minus_s $mailuser\@mail.cern.ch";
            open(M,"|$open") || Die("cannot open pipe for $open");
            print M "Subject: $sub\n" unless $minus_s;
        }
        
        &Log("send 'The cron command\n     $reply\nproduced the following output:'");
        print M "The cron command\n     $reply\nproduced the following output:\n";
        
        seek($TMP,0,0);
        while (<$TMP>) {print M $_;}
        close($TMP);
        close(M);        
        &Log("$proc msg $?");
        #unlink($tmp);                  # keep evidences
    } else {        
        close($TMP);
        unlink($tmp);        
    }        

        
# clear any ticket file
    &clean;

}        



sub clean {
    $< = $uid;         # uid=user, euid=root
    if ( $krb5file && -O "$krb5file" ) { 
        &Log("remove ticket file \"$krb5file\" size " . -s($krb5file)) if $Debug;
        unlink($krb5file); 
    }
    $< = 0;            #  totally root again
}

sub Log {
    &log("$$: @_");
}
sub Die {
    &Log("@_");
    die "@_\n";
}
