# ARC - Authenticated Remote Control
# common routines and command interpreter
#
# Author: R.Toebbicke, CERN
#

$|=1;
@ARCETC=("/usr/libexec/arcd") unless defined(@ARCETC);

chop($ENV{"SCLHOST"}=`uname -n`);
if ($ENV{"SCPRINCIPAL"}=~/^([^\.]+)\.([^\@]*)\@(.*)/) {
	$ENV{"SCUSER"}=$1;
	$ENV{"SCINSTANCE"}=$2;
	$ENV{"SCREALM"}=$3;
}
$ENV{"SCACL"}=($ENV{"SCPRINCIPAL"} eq "rcmd.$ENV{SCLHOST}\@$ENV{SCLREALM}")
		|| &aclcheck($ENV{"SCPRINCIPAL"});

# Check the logfile
# 11/10/2001 Vladimir.Bahyl@cern.ch
foreach $etc (@ARCETC) {
  if (-r "$etc/config.arc" && -f "$etc/config.arc" && -s "$etc/config.arc") {
    require "$etc/config.arc";
    last;
  }
}

$USERARG=pop @ARGV;
foreach (@ARGV) {	# Execute remaining constant arguments
	eval $_;
}

# Some predefined commands
$ACL{'whoami'}="ANY"; sub whoami { print $ENV{"SCPRINCIPAL"},"\n";}
$ACL{'uptime'}="ANY"; sub uptime { system "uptime"; }
$ACL{'checkv'}="AUTH"; sub checkv { system "/usr/afsws/bin/fs checkv"; }



if ( $USERARG ne '') {
	&exec_proc($USERARG);
} else {
	while(<STDIN>) {
		if (!/^\s*\#/) { &exec_proc($_);}
	}
}

1;

# &aclcheck($userid,$ACL-file);
sub aclcheck {
	if ($ENV{"SCACL"}) { return 1; }

	local($user,$acl)=@_;
	local($_,$ETC,$fname);

	$acl="ACL" unless $acl;

	if ($acl eq "ANY") { return 1; }
	if ($acl eq "AUTH") { return (index($user,"\@")!=$[); }

	if ($acl eq 'ACL') {
		$fname="$ARCETC[0]/ACL";
	} elsif (substr($acl,$[,1) eq "/") {	# absolute path
		$fname=$acl;
	} else {
		foreach $ETC (@ARCETC) {
			$fname="$ETC/$acl.acl";
			last if ( -r $fname);
		}
	}
	open(A,$fname) || return(0);
	$user.="\n";
	while (<A>) {
		if ($_ eq $user) {
			close(A); return 1;
		}
	}
	close(A);
	return(0);
}


sub exec_proc {
	local($_)=@_;
	local($proc,$args,$ETC);

	if (/^([a-zA-Z0-9_-]+)\s*/) {
		$proc=$1;
		$args=$';
		$proc=~s/-/_/g;
	} else { die "Unknown command: $_"; }

	if (!$ENV{"SCACL"}) {   # check for tricks in command
		&safeargs($proc);
	}

	# If procedure not (yet) defined, try require first
	if (!defined(&$proc)) {
		foreach $etc (@ARCETC) {
			if (-r "$etc/$proc.arc") {
				$ETC=$etc;
				require "$ETC/$proc.arc";
				last;
			}
		}
	}

	if (defined(&$proc) && &aclcheck($ENV{"SCPRINCIPAL"},$ACL{$proc})) {
		&$proc($args);
	} elsif ($ENV{"SCACL"}) {
		eval $_; die "$@" if $@;
	} else { die "unauthorized access for '$proc'";}
}

sub safeargs {
	if ("@_"=~/[;`\&\|]/) { die "Invalid arguments"; }
}

sub optargs {
	local($_)=@_;
	local($x,$s);
	s/^\s+//;	# skip leading blanks even if no parms follow
	while($_) {
		if (/^\s*-(\w)(\S*)\s*/) {
			$x= ($2 eq '') ? 1 : $2;
			eval "\$opt_$1=\$x";
			$_=$';
		} else { last; }
        }
	return($_);
}

sub log {
  if (defined $USE_SYSLOG) {
    # 5/10/2001 - by Vladimir.Bahyl@cern.ch
    use lib '/usr/sue/dist/lib/sue';
    require Syslog;
    syslog(undef, "info|authpriv", "arc[$$]: $ENV{SCHOST} $ENV{SCPRINCIPAL} $_[0]");
  } else {
	if (!defined($ARCLOG)) {
		if (-d "/var/adm") { $ARCLOG="/var/adm/arc.log"; }
		else {$ARCLOG="/usr/adm/arc.log"; }
	}
	local($now); $now=localtime();
	if (! -r $ARCLOG) { open(L,">$ARCLOG"); }
	open(L,">>$ARCLOG") || return;
	printf(L "%s: $ENV{SCHOST} $ENV{SCPRINCIPAL} $_[0]\n", $now);
	close(L);
  }
}
