#!/usr/bin/perl
# ARC - Authenticated Remote Control
# common routines and command interpreter
#
# Author: R.Toebbicke, CERN
#
$ACL{'help'}="ANY";

sub help {
	print <<EOFHELP
Available commands:
	whoami
	uptime
	...
EOFHELP
;
}

@ARCETC=("/usr/libexec/arcd");
$|=1;

chop($ENV{"SCLHOST"}=`uname -n`);
if ($ENV{"SCPRINCIPAL"}=~/^([^\.@\/]+)[\.\/]?([^\@]*)\@(.*)/) {
	$ENV{"SCUSER"}=$1;
	$ENV{"SCINSTANCE"}=$2;
	$ENV{"SCREALM"}=$3;
} 

# Check the logfile
# 11/10/2001 Vladimir.Bahyl@cern.ch
foreach $etc (@ARCETC) {
  if (-r "$etc/config.arc" && -f "$etc/config.arc" && -s "$etc/config.arc") {
    require "$etc/config.arc";
    last;
  }
}

$USERARG=pop @ARGV;
foreach (@ARGV) {	# Execute remaining constant arguments
	eval $_;
}

$ENV{'PATH'}.=":/usr/afsws/bin";

# Some predefined commands
$ACL{'whoami'}="ANY"; sub whoami { print $ENV{"SCPRINCIPAL"},"\n";}
$ACL{'uptime'}="ANY"; sub uptime { system "uptime"; }
$ACL{'checkv'}="AUTH"; sub checkv { system "fs checkvolumes"; }

if ( $USERARG ne '') {
	&exec_proc($USERARG);
} else {
	while(<STDIN>) {
		if (!/^\s*\#/) { &exec_proc($_);}
	}
}

1;

# &aclcheck($userid,$ACL-file);
sub aclcheck {
    return &aclcheck_f(@_) || &aclcheck_f(@_[0]);
}

sub aclcheck_f {
	local($user,$acl)=@_;
	local($_,$ETC,$fname);


	if ($acl eq "ANY") { return 1; }
	if ($acl eq "AUTH") { return (index($user,"\@")!=$[); }

	$acl="ACL" unless $acl;
	if ($acl eq 'ACL') {
	    return 1 if $user eq "host/$ENV{SCLHOST}\@$ENV{SCLREALM}";
	    foreach $d (@ARCETC) {
		$fname="$d/ACL";
		last if -r $fname;
	    }
	} elsif (substr($acl,$[,1) eq "/") {	# absolute path
		$fname=$acl;
	} else {
		foreach $ETC (@ARCETC) {
			$fname="$ETC/$acl.acl";
			last if ( -r $fname);
		}
	}
	open(A,$fname);
	local(@a)=<A>;
	close(A);

	return 1 if grep(/^\Q$user\E$/, @a);
	$user=~s/\@/.\@/;	# tribute to krb4...
	return 1 if grep(/^\Q$user\E$/, @a);
	return(0);
}


sub exec_proc {
	local($_)=@_;
	local($proc,$args,$ETC);
	local($super) = &aclcheck($ENV{"SCPRINCIPAL"});

	if (/^([a-zA-Z0-9_-]+)\s*/) {
		$proc=$1;
		$args=$';
		$proc=~s/-/_/g;
	} else { die "Unknown command: $_"; }

	unless ($super) {   # check for tricks in command
		&safeargs($proc);
	}

	# If procedure not (yet) defined, try require first
	if (!defined(&$proc)) {
		foreach $etc (@ARCETC) {
			if (-r "$etc/$proc.arc") {
				$ETC=$etc;
				require "$ETC/$proc.arc";
				last;
			}
		}
	}

	if (defined(&$proc) && &aclcheck($ENV{"SCPRINCIPAL"},$ACL{$proc})) {
		&$proc($args);
	} elsif ($super) {
		eval $_; die "$@" if $@;
	} else {
	    &log("unauthorized access for '$proc' by $ENV{'SCPRINCIPAL'} on $ENV{'SCHOST'}");
	    die "unauthorized access for '$proc'";
	}
}

sub safeargs {
	if ("@_"=~/[;`\&\|]/) { die "Invalid arguments"; }
}

sub optargs {
	local($_)=@_;
	local($x,$s);
	s/^\s+//;	# skip leading blanks even if no parms follow
	while($_) {
		if (/^\s*-(\w)(\S*)\s*/) {
			$x= ($2 eq '') ? 1 : $2;
			eval "\$opt_$1=\$x";
			$_=$';
		} else { last; }
        }
	return($_);
}

sub log {
  if (defined $USE_SYSLOG) {
    # 5/10/2001 - by Vladimir.Bahyl@cern.ch
    use lib '/usr/sue/dist/lib/sue';
    require Sys::Syslog;
    Sys::Syslog::syslog(undef, "info|authpriv", "arc[$$]: $ENV{SCHOST} $ENV{SCPRINCIPAL} $_[0]");
  } else {
	if (!defined($ARCLOG)) {
	        foreach $d ("/var/log", "/var/adm") {
		    if (-d $d) { $ARCLOG="$d/arc.log"; last; }
		}
	}
	if (!defined($ARCLOG)) {
		$ARCLOG="/usr/adm/arc.log";
	}
	local($now); $now=localtime();
	if (! -r $ARCLOG) { open(L,">$ARCLOG"); }
	open(L,">>$ARCLOG") || return;
	printf(L "%s: $ENV{SCHOST} $ENV{SCPRINCIPAL} $_[0]\n", $now);
	close(L);
  }
}
